'use strict';

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var React = _interopDefault(require('react'));
var mdxDeck = require('mdx-deck');
var standalone = require('@code-surfer/standalone');
var shellQuote = require('shell-quote');
var useSpring = require('use-spring');

function Step() {
  return null;
}

function _extends() {
  _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  return _extends.apply(this, arguments);
}

function _inheritsLoose(subClass, superClass) {
  subClass.prototype = Object.create(superClass.prototype);
  subClass.prototype.constructor = subClass;
  subClass.__proto__ = superClass;
}

/**
 * The metastring is the thing that comes after the language in markdown codeblocks
 *
 * ```js this is the metastring
 * code goes here
 * ```
 */

function parseMetastring(metastring) {
  if (!metastring) {
    return {};
  }

  var argv = shellQuote.parse(metastring);
  var result = {};
  argv.forEach(function (arg) {
    if (!arg.includes("=")) {
      result.focus = arg;
    } else {
      var _arg$split = arg.split(/=(.*)/),
          key = _arg$split[0],
          value = _arg$split[1];

      result[key] = value;
    }
  });
  return result;
}

function readStepFromElement(element) {
  if (!element.props.children || !element.props.children.props) {
    return null;
  }

  var props = element.props.children.props;
  var className = props.className;
  return _extends({
    code: props.children,
    lang: className && className.substring("language-".length)
  }, parseMetastring(props.metastring));
}

var ErrorBoundary =
/*#__PURE__*/
function (_React$Component) {
  _inheritsLoose(ErrorBoundary, _React$Component);

  function ErrorBoundary(props) {
    var _this;

    _this = _React$Component.call(this, props) || this;
    _this.state = {};
    return _this;
  }

  ErrorBoundary.getDerivedStateFromError = function getDerivedStateFromError(error) {
    return {
      error: error
    };
  };

  var _proto = ErrorBoundary.prototype;

  _proto.componentDidCatch = function componentDidCatch(error, info) {// console.log(error, info);
  };

  _proto.render = function render() {
    if (!this.state.error) {
      return this.props.children;
    } else if (this.state.error.element) {
      return this.state.error.element;
    } else {
      console.error(this.state.error);
      return React.createElement(standalone.UnknownError, {
        error: this.state.error
      });
    }
  };

  return ErrorBoundary;
}(React.Component);

function useNotes(notesElements) {
  var context = mdxDeck.useDeck();
  React.useEffect(function () {
    if (!context || !context.register) return;
    if (typeof context.index === "undefined") return;
    var notes = getNotesFromElements(notesElements);
    context.register(context.index, {
      notes: notes
    });
  }, []);
}

function getNotesFromElements(notesElements) {
  var notes = notesElements.map(function (element) {
    if (!element) {
      // this is a step with empty notes
      return null;
    }

    var props = element.props;

    if (props.inline) {
      // this is <Notes inline={true} />
      return {
        inline: true,
        text: props.children
      };
    } // this is <Notes>something</Notes>
    // we shouldn't return an object here,
    // to be compatible with the default Presenter


    return props && props.children;
  });

  if (notes.length) {
    var lastNotes = notes[notes.length - 1]; // we add an extra EOL to the last step

    notes[notes.length - 1] = (lastNotes || "") + "\n";
  }

  return notes;
}

function useSteps (stepsCount) {
  var step = mdxDeck.useSteps(stepsCount);
  return step === Infinity ? 0 : step;
}

function useStepSpring(stepsCount) {
  // step index according to mdx-deck
  var targetStepIndex = useSteps(stepsCount - 1); // real number between 0 and stepsCount - 1

  var currentStepSpring = useSpring.useSpring(targetStepIndex, {
    decimals: 3,
    stiffness: 80,
    damping: 48,
    mass: 8
  });
  return currentStepSpring;
}

function CodeSurferLayout(_ref) {
  var children = _ref.children;
  var deck = mdxDeck.useDeck();
  var steps = React.useMemo(getStepsFromChildren(children), [deck.index]);
  useNotes(steps.map(function (s) {
    return s.notesElement;
  }));
  var progress = useStepSpring(steps.length);
  return React.createElement("div", {
    style: {
      width: "100vw",
      maxWidth: "100%",
      height: "100vh",
      display: "flex",
      flexDirection: "column",
      justifyContent: "center",
      alignItems: "center"
    },
    className: "cs-layout"
  }, React.createElement(standalone.CodeSurfer, {
    steps: steps,
    progress: progress
  }));
}

var getStepsFromChildren = function getStepsFromChildren(children) {
  return function () {
    var kids = React.Children.toArray(children);
    return kids.map(function (child, i) {
      var step = readStepFromElement(child);
      if (!step) return;
      var nextChild = kids[i + 1];

      if (nextChild && nextChild.props && nextChild.props.originalType === mdxDeck.Notes) {
        step.notesElement = nextChild;
      }

      return step;
    }).filter(function (x) {
      return x;
    });
  };
};

var layout = (function (props) {
  return React.createElement(ErrorBoundary, null, React.createElement(CodeSurferLayout, Object.assign({}, props)));
});

function ColumnLayout(_ref) {
  var children = _ref.children,
      _ref$themes = _ref.themes,
      themes = _ref$themes === void 0 ? [] : _ref$themes,
      sizes = _ref.sizes;
  var deck = mdxDeck.useDeck();

  var _React$useMemo = React.useMemo(function () {
    return getColumnsFromChildren(children, sizes);
  }, [deck.index]),
      columns = _React$useMemo[0],
      titles = _React$useMemo[1],
      subtitles = _React$useMemo[2],
      notesElements = _React$useMemo[3];

  useNotes(notesElements);
  var progress = useStepSpring(columns[0].steps.length);
  var stepIndex = Math.round(progress);
  return React.createElement("div", {
    style: {
      width: "100vw",
      maxWidth: "100%",
      height: "100vh",
      display: "flex",
      flexDirection: "row",
      justifyContent: "center",
      alignItems: "center",
      fontSize: "0.8em",
      position: "relative"
    },
    className: "cs-col-layout"
  }, columns.map(function (column, i) {
    return React.createElement(Column, {
      key: i,
      column: column,
      progress: progress,
      theme: themes[i]
    });
  }), React.createElement(standalone.StylesProvider, null, React.createElement(Title, {
    text: titles[stepIndex]
  }), React.createElement(Subtitle, {
    text: subtitles[stepIndex]
  })));
}

function Column(_ref2) {
  var column = _ref2.column,
      progress = _ref2.progress,
      theme = _ref2.theme;
  return React.createElement("div", {
    style: {
      flex: column.flex,
      overflow: "hidden",
      height: "100%"
    }
  }, column.isCode ? React.createElement(standalone.CodeSurfer, {
    steps: column.steps,
    progress: progress,
    theme: theme
  }) : column.steps[Math.round(progress)].element);
}

function Title(_ref3) {
  var text = _ref3.text;
  if (!text) return null;
  return React.createElement(standalone.Styled.Title, {
    className: "cs-title"
  }, React.createElement("span", null, text));
}

function Subtitle(_ref4) {
  var text = _ref4.text;
  if (!text) return null;
  return React.createElement(standalone.Styled.Subtitle, {
    className: "cs-subtitle",
    style: {
      margin: "0.3em 0"
    }
  }, React.createElement("span", null, text));
}

function getColumnsFromChildren(children, sizes) {
  if (sizes === void 0) {
    sizes = [];
  }

  var columns = [];
  var stepElements = React.Children.toArray(children);
  stepElements.forEach(function (stepElement, stepIndex) {
    React.Children.toArray(stepElement.props.children).filter(function (element) {
      return element.props && element.props.originalType !== mdxDeck.Notes;
    }).forEach(function (codeElement, columnIndex) {
      columns[columnIndex] = columns[columnIndex] || {
        steps: [],
        isCode: true
      };
      var step = readStepFromElement(codeElement);
      columns[columnIndex].isCode = columns[columnIndex].isCode && step;
      columns[columnIndex].steps[stepIndex] = step || {
        element: codeElement
      };
    });
  });
  columns.forEach(function (column, columnIndex) {
    column.flex = sizes[columnIndex] || 1;
  });
  var titles = stepElements.map(function (stepElement) {
    return stepElement.props.title;
  });
  var subtitles = stepElements.map(function (stepElement) {
    return stepElement.props.subtitle;
  });
  var notesElements = stepElements.map(function (stepElement) {
    var stepChildren = React.Children.toArray(stepElement.props.children);
    var notesElement = stepChildren.find(function (element) {
      return element.props && element.props.originalType === mdxDeck.Notes;
    });
    return notesElement;
  });
  return [columns, titles, subtitles, notesElements];
}

var columnLayout = (function (props) {
  return React.createElement(ErrorBoundary, null, React.createElement(ColumnLayout, Object.assign({}, props)));
});

function mdThemes(theme) {
  // const stringStyle = theme.styles.find(s => s.types.includes("string"));
  // const color = theme.pre.color;
  // const bg = theme.pre.background;
  // return [
  //   { codeSurfer: theme },
  //   {
  //     colors: {
  //       text: color,
  //       background: bg,
  //       link: stringStyle && stringStyle.style.color,
  //       pre: color,
  //       code: color,
  //       preBackground: bg
  //     },
  //     Presenter,
  //     codeSurfer: theme
  //   }
  // ];
  return [theme, theme];
}

var _mdThemes =
/*#__PURE__*/
mdThemes(standalone.dracula),
    draculaPartial = _mdThemes[0],
    dracula = _mdThemes[1];

var _mdThemes2 =
/*#__PURE__*/
mdThemes(standalone.duotoneLight),
    duotoneLightPartial = _mdThemes2[0],
    duotoneLight = _mdThemes2[1];

var _mdThemes3 =
/*#__PURE__*/
mdThemes(standalone.duotoneDark),
    duotoneDarkPartial = _mdThemes3[0],
    duotoneDark = _mdThemes3[1];

var _mdThemes4 =
/*#__PURE__*/
mdThemes(standalone.github),
    githubPartial = _mdThemes4[0],
    github = _mdThemes4[1];

var _mdThemes5 =
/*#__PURE__*/
mdThemes(standalone.nightOwl),
    nightOwlPartial = _mdThemes5[0],
    nightOwl = _mdThemes5[1];

var _mdThemes6 =
/*#__PURE__*/
mdThemes(standalone.oceanicNext),
    oceanicNextPartial = _mdThemes6[0],
    oceanicNext = _mdThemes6[1];

var _mdThemes7 =
/*#__PURE__*/
mdThemes(standalone.shadesOfPurple),
    shadesOfPurplePartial = _mdThemes7[0],
    shadesOfPurple = _mdThemes7[1];

var _mdThemes8 =
/*#__PURE__*/
mdThemes(standalone.ultramin),
    ultraminPartial = _mdThemes8[0],
    ultramin = _mdThemes8[1];

var _mdThemes9 =
/*#__PURE__*/
mdThemes(standalone.vsDark),
    vsDarkPartial = _mdThemes9[0],
    vsDark = _mdThemes9[1];

exports.CodeSurferColumnLayout = columnLayout;
exports.CodeSurferLayout = layout;
exports.Step = Step;
exports.dracula = dracula;
exports.draculaPartial = draculaPartial;
exports.duotoneDark = duotoneDark;
exports.duotoneDarkPartial = duotoneDarkPartial;
exports.duotoneLight = duotoneLight;
exports.duotoneLightPartial = duotoneLightPartial;
exports.github = github;
exports.githubPartial = githubPartial;
exports.nightOwl = nightOwl;
exports.nightOwlPartial = nightOwlPartial;
exports.oceanicNext = oceanicNext;
exports.oceanicNextPartial = oceanicNextPartial;
exports.shadesOfPurple = shadesOfPurple;
exports.shadesOfPurplePartial = shadesOfPurplePartial;
exports.ultramin = ultramin;
exports.ultraminPartial = ultraminPartial;
exports.vsDark = vsDark;
exports.vsDarkPartial = vsDarkPartial;
//# sourceMappingURL=code-surfer.cjs.development.js.map
